/*
	$Id: DaemonComm.h 1211 2018-11-28 20:09:51Z john $

	This file is part of the Base4c Foundation Classes.

	Base4c is Copyright (c) 2010 - 2018, The 4c Company.

	Base4c is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	Base4c is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __Base4c_DaemonComm_h__
#define __Base4c_DaemonComm_h__

#include <vector>
#include <string>
#include <Base4c/Mutex.h>

namespace Base4c
{
	class Base4c_EXPORT DaemonComm
	{
		public:
			typedef enum { pingSucceeded, pingFailed } PingStatus;
			typedef enum { error = -1, response = 0, unsolicited = 1 } Status;

		private:
			int _s;
			bool _bRunning;
			bool _bStop;
#if defined (_WINDOWS)
			static bool _bWSStarted;
#endif /* _WINDOWS */
			bool _bIsConnected;
			bool _bParsedOk;
			void(*_usmCallback)( Status status, const std::string& strMessage );
			Base4c::Mutex _mxStop;
			std::string _strHost;
			int _nPort;
			PingStatus _pingStatus;
			std::string _strProtocol;
			std::string _strUser;
			std::string _strPassword;

		public:
			DaemonComm( void(*usmCallback)( Status status, const std::string& strMessage ) );
			virtual ~DaemonComm();

			const std::string& host() const { return _strHost; }
			int port() const { return _nPort; }
			const std::string& protocol() const { return _strProtocol; }
			bool isConnected() const { return _bIsConnected; }
			bool parsedOk() const { return _bParsedOk; }
			/*
				parse (0)
				The old original version parses the connect string
			*/
			virtual bool parse( const std::string& strConnect );
			/*
				parse (1)
				New version for UBERS/CAP that will probably go out the window
			*/
			virtual bool parse( std::string& strProtocol, std::string& strHost, int& nPort, const std::string& strConnect );
			/*
				parse(2)
				Even newer version:
				Parses the connect string. Ensures that the protocol matches that provided alse returns false
				returns host, port, user id and password
				assumes connect string is in the form <user_id>:<password>@<protocol>://<host>:<port>
			*/
			virtual bool parse( std::string& strUser, std::string& strPassword, std::string& strHost, int& nPort, const std::string& strConnect, const std::string& strProtocol );
			bool connect();
			bool disconnect();

			bool createPingRequest();
			bool send( const std::string& str );

			PingStatus pingStatus() const { return _pingStatus; }

		protected:
#if defined (_WINDOWS)
			static void responseThread( void* pvContext );
#elif defined (LINUX)
			static void* responseThread( void* pvContext );
#endif /* _WINDOWS */
			void processStream( const std::string& strStream );
			virtual void processMessage( const std::string& strMessage );
	};
};

#endif /* !__Base4c_DaemonComm_h__ */
