/*
	$Id: Date.h 1283 2020-05-10 14:35:16Z john $

	This file is part of the Base4c Foundation Classes.

	Base4c is Copyright (c) 2010 - 2018, The 4c Company.

	Base4c is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	Base4c is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __Base4c_Date_h__
#define __Base4c_Date_h__

#include <Base4c/Base4c.h>
#include <Base4c/Exception.h>

#include <string>

namespace Base4c
{
	class Base4c_EXPORT Date
	{
		public:
			static const int nINVALID_DATE = 1;
			static const Date Forever;
			static const Date Never;
			enum DayOfWeek { Unknown = -1,
					Sunday = 0,
					Monday,
					Tuesday,
					Wednesday,
					Thursday,
					Friday,
					Saturday };

		private:
			int _nYear;
			int _nMonth;
			int _nDay;

		public:
			Date();
			Date( int nYear, int nMonth, int nDay );
			Date( const Date& date );
			virtual ~Date();

			bool isValid() const;
			bool isForever() const;

			int year() const;
			int month() const;
			int day() const;

			Date lastDay() const;
			DayOfWeek dayOfWeek() const;

			const Date& operator=( const Date& date );

			bool operator==( const Date& date ) const;
			bool operator!=( const Date& date ) const;
			bool operator<( const Date& date ) const;
			bool operator>( const Date& date ) const;
			bool operator<=( const Date& date ) const;
			bool operator>=( const Date& date ) const;

			static Date fromText( const std::string& strText );
			std::string asText() const;

			static const Date now( Zone zone = local );
	};
};

#endif /* !__Base4c_Date_h__ */
