/*
	$Id: EventLogger.h 1239 2018-12-03 15:51:01Z john $

	This file is part of the Base4c Foundation Classes.

	Base4c is Copyright (c) 2010 - 2018, The 4c Company.

	Base4c is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	Base4c is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __Base4c_EventLogger_h__
#define __Base4c_EventLogger_h__

#include <Base4c/Base4c.h>

#include <string>

namespace Base4c
{
	class Exception;

	class Base4c_EXPORT EventLogger
	{
		public:
			typedef enum { dal, data, extract, comms, ui, daemon } Layer;
			typedef enum { normal, high, debug, trace } Level;

		private:
			static EventLogger* _plogger;
#if defined (_WINDOWS)
			HANDLE _hEventLog;
			HMODULE _hModule;
#endif
			Level mlevel;

		public:
			EventLogger( const std::string& strEventSource );
			virtual ~EventLogger();

			static EventLogger* theLogger();

// Errors
			bool log( const Base4c::Exception& e, Layer layer );
			bool logUnknown( Layer layer );
			bool logError( const std::string& strError, Layer layer );
// Warning
			bool logWarning( const std::string& strWarning, Layer layer );
// Message
			bool logMessage( const std::string& strMessage, Layer layer )
			{
				if( mlevel >= high )
					return logMessageP( strMessage, layer );
				else
					return false;
			}
// Debug
			bool logDebug( const std::string& strDebug, Layer layer )
			{
				if( mlevel >= debug )
					return logDebugP( strDebug, layer );
				else
					return false;
			}
// Trace
			bool logTrace( const std::string& strTrace, Layer layer )
			{
				if( mlevel >= trace )
					return logTraceP( strTrace, layer );
				else
					return false;
			}

			bool registerEventSource( const std::string& strSource );
#if defined (_WINDOWS)
			void setModuleHandle( HMODULE handle );
#endif
			Level level() const { return mlevel; }
			void setLevel( Level level ){ mlevel = level; }

		protected:
// Message
			bool logMessageP( const std::string& strMessage, Layer layer );
// Debug
			bool logDebugP( const std::string& strDebug, Layer layer );
// Trace
			bool logTraceP( const std::string& strTrace, Layer layer );
	};
};

#endif /* !__Base4c_EventLogger_h__ */
