/*
	$Id: Exception.h 1211 2018-11-28 20:09:51Z john $

	This file is part of the Base4c Foundation Classes.

	Base4c is Copyright (c) 2010 - 2018, The 4c Company.

	Base4c is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	Base4c is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __Base4c_Exception_h__
#define __Base4c_Exception_h__

#include <Base4c/Base4c.h>

#include <string>

namespace Base4c
{
	const int B4CX_SEELEGEND = 0;
	const int B4CX_MALFORMEDEXCEPTION = 1;
	const int B4CX_UNKNOWN = 2;
	const int B4CX_NOT_IMPLEMENTED = 3;
	const int B4CX_DAEMONCONNECTFAILED = 1001;
	const int B4CX_DAEMONNOTCONNECTED = 1002;
	const int B4CX_DAEMONAUTHREJECT = 1003;
	const int B4CX_MALFORMEDXML = 2001;
	const int B4CX_MALFORMEDROW = 2002;
	const int B4CX_BADTEMPLATE = 3001;
	const int B4CX_NOSUCHVARIABLE = 3002;
	const int B4CX_NOCREATEPATH = 4001;
	const int B4CX_SOLOADFAILED = 4100;
	const int B4CX_SYMLINKFAILED = 4101;
	const int B4CX_SEMWAITFAILED = 4200;
	const int B4CX_SEMUNKNOWN = 4201;
	const int B4CX_INVALIDTYPE = 4300;
	const int B4CX_TYPEMISMATCH = 4301;
	const int B4CX_COERSIONFAILED = 4302;
	const int B4CX_DERIVEDBASE = 10000; // Derived exceptions should start above this. Base4c Exceptions should be below this.

	class Base4c_EXPORT Exception
	{
		private:
			std::string _strFile;
			int _nLine;
			int _nErrorCode;
			bool _bFatal;
			std::string _strLegend;

		public:
			Exception( int nErrorCode, bool bFatal = false, const std::string& strLegend = std::string() );
			Exception( const std::string& strFile, int nLine, int nErrorCode, bool bFatal = false, const std::string& strLegend = std::string() );
			Exception( const Exception& exception );
			virtual ~Exception();

			const Exception& operator=( const Exception& exception );

			const std::string& file() const;
			int line() const;
			int errorCode() const;
			bool isFatal() const;
			const std::string& legend() const;

			static const std::string explainErrorCode( int nErrorCode );

		protected:
			/**
			 * DO NOT call the default constructor. This is only for use in the fromXML code. The default
			 * constructed exception will be malformed and fatal. YOU HAVE BEEN WARNED.
			 */
			Exception();
	};
};

Base4c_EXPORT std::ostream& operator<<( std::ostream& stream, const Base4c::Exception& e );

#endif /* !__Base4c_Exception_h__ */
