/*
	$Id: Guid.h 1211 2018-11-28 20:09:51Z john $

	This file is part of the Base4c Foundation Classes.

	Base4c is Copyright (c) 2010 - 2018, The 4c Company.

	Base4c is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	Base4c is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __Base4c_Guid_h__
#define __Base4c_Guid_h__

#include <Base4c/Base4c.h>

#if defined (LINUX)
#	include <uuid/uuid.h>
#endif /* LINUX */

#if defined (LINUX)
	typedef uuid_t GUID_T;
#elif defined (_WINDOWS)
	typedef GUID GUID_T;
#endif /* LINUX */

#include <string>

namespace Base4c
{
	class Base4c_EXPORT Guid
	{
		private:
			GUID_T _guid;

		public:
			static const Guid null;

		public:
			Guid();
			Guid( const GUID_T& guid );
			Guid( const Guid& guid );
			virtual ~Guid();

			const Guid& operator=( const Guid& guid );

			bool operator==( const Guid& guid ) const;
			bool operator!=( const Guid& guid ) const;
			bool operator<( const Guid& guid ) const;
			bool operator>( const Guid& guid ) const;

			const GUID_T& asGuid() const;
			std::string asString() const;

			static Guid create();
			static Guid fromString( const std::string& str );
	};
};

#endif /* !__Base4c_Guid_h__ */
