/*
	$Id: Mutex.h 1211 2018-11-28 20:09:51Z john $

	This file is part of the Base4c Foundation Classes.

	Base4c is Copyright (c) 2010 - 2018, The 4c Company.

	Base4c is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	Base4c is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __Base4c_Mutex_h__
#define __Base4c_Mutex_h__

#include <Base4c/Base4c.h>

#include <string>

#if defined (LINUX)
#	include <pthread.h>
#endif

namespace Base4c
{
	class Base4c_EXPORT Mutex
	{
		public:
			class Base4c_EXPORT Lock
			{
				private:
					Mutex& _mx;

				public:
					Lock( Mutex& mx, const char* pszReason );
					virtual ~Lock();
			};

		private:
#if defined (LINUX)
			pthread_mutex_t _mx;
			pthread_t _lockedBy;
#elif defined (_WINDOWS)
			CRITICAL_SECTION _cs;
#endif
			char* _pszName;

			Mutex();
			Mutex( const Mutex& ){}
			const Mutex& operator=( const Mutex& ){ return *this; }

		public:
			Mutex( const char* pszName );
			virtual ~Mutex();

			void lock( const char* pszReason );
			void unlock();
	};
};

#endif /* !__Base4c_Mutex_h__ */
