/*
	$Id: Publisher.h 1211 2018-11-28 20:09:51Z john $

	This file is part of the Base4c Foundation Classes.

	Base4c is Copyright (c) 2010 - 2018, The 4c Company.

	Base4c is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	Base4c is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __Base4c_Publisher_h__
#define __Base4c_Publisher_h__

#include <Base4c/Base4c.h>

#include <Base4c/Mutex.h>

#include <set>

/*
	NOTE: When a subscriber is notified that a publisher to which it is
	subscribed is being deleted, it MUST NOT attempt to remove itself as
	a subscriber as this will cause a deadlock.
*/

namespace Base4c
{
	class Subscriber;

	class Base4c_EXPORT Publisher
	{
		public:
			enum Notification{ notifyChanged, notifyDeleting };

		private:
			Mutex _mx;
			bool _bIsDeleting;
			std::set<Subscriber*> _subscriberS;

		public:
			Publisher();
			Publisher( const Publisher& publisher );
			virtual ~Publisher();

			const Publisher& operator=( const Publisher& publisher );

			void addSubscriber( Subscriber* psubscriber );
			void removeSubscriber( Subscriber* psubscriber );

			const std::set<Subscriber*>& subscribers() const;

			void changed( void* pnotifier = NULL );
	};
};

#endif /* !__Base4c_Publisher_h__ */
