/*
	$Id: Subscriber.h 1211 2018-11-28 20:09:51Z john $

	This file is part of the Base4c Foundation Classes.

	Base4c is Copyright (c) 2010 - 2018, The 4c Company.

	Base4c is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	Base4c is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __Base4c_Subscriber_h__
#define __Base4c_Subscriber_h__

#include <Base4c/Base4c.h>
#include <Base4c/Publisher.h>

/*
	NOTE: It is the responsibility of the client of this class to ensure that,
	if the subscriber context is used, it points to an object/entity whose
	lifespan will exceed that of the subscriber.
*/

namespace Base4c
{
	class Base4c_EXPORT Subscriber
	{
		private:
			void* _pvSubscriberContext;
			bool _bNotifyDeletes;

		public:
			Subscriber( bool bNotifyDeletes = true );
			Subscriber( void* pvSubscriberContext, bool bNotifyDeletes = true );
			Subscriber( const Subscriber& subscriber );
			virtual ~Subscriber();

			const Subscriber& operator=( const Subscriber& subscriber );

			const void* subscriberContext() const;
			bool notifyDeletes() const;
			virtual void notify( Publisher* ppublisher, Publisher::Notification notification, void* pnotifier ) = 0;
	};
};

#endif /* !__Base4c_Subscriber_h__ */
