/*
	$Id: Time.h 1239 2018-12-03 15:51:01Z john $

	This file is part of the Base4c Foundation Classes.

	Base4c is Copyright (c) 2010 - 2018, The 4c Company.

	Base4c is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	Base4c is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __Base4c_Time_h__
#define __Base4c_Time_h__

#include <Base4c/Base4c.h>
#include <Base4c/Exception.h>

#include <string>

namespace Base4c
{
	class Base4c_EXPORT Time
	{
		public:
			static const int nINVALID_TIME = 2;

		private:
			int _nHour;
			int _nMinute;
			double _dSecond;

		public:
			Time();
			Time( int nHour, int nMinute, double dSecond );
			Time( const Time& time );
			virtual ~Time();

			bool isValid() const;

			int hour() const;
			int minute() const;
			double second() const;

			const Time& operator=( const Time& time );

			bool operator==( const Time& time ) const;
			bool operator!=( const Time& time ) const;
			bool operator<( const Time& time ) const;
			bool operator>( const Time& time ) const;
			bool operator<=( const Time& time ) const;
			bool operator>=( const Time& time ) const;

			static Time fromText( const std::string& strText );
			std::string asText() const;

			static const Time now( Zone zone = local );
	};
};

#endif /* !__Base4c_Time_h__ */
