/*
	$Id: Timestamp.h 1239 2018-12-03 15:51:01Z john $

	This file is part of the Base4c Foundation Classes.

	Base4c is Copyright (c) 2010 - 2018, The 4c Company.

	Base4c is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	Base4c is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __Base4c_Timestamp_h__
#define __Base4c_Timestamp_h__

#include <Base4c/Base4c.h>
#include <Base4c/Exception.h>

#include <Base4c/Date.h>
#include <Base4c/Time.h>

#include <string>

namespace Base4c
{
	class Base4c_EXPORT Timestamp : public Date, public Time
	{
		public:
			static const int nINVALID_TIMESTAMP = 3;
			static const Timestamp Forever;
			static const Timestamp Never;

		private:

		public:
			Timestamp();
			Timestamp( int nYear, int nMonth, int nDay, int nHour, int nMinute, double dSecond );
			Timestamp( const Timestamp& timestamp );
			Timestamp( const Date& date, const Time& time );
			virtual ~Timestamp();

			bool isValid() const;
			bool isForever() const;

			const Timestamp& operator=( const Timestamp& timestamp );

			bool operator==( const Timestamp& timestamp ) const;
			bool operator!=( const Timestamp& timestamp ) const;
			bool operator<( const Timestamp& timestamp ) const;
			bool operator>( const Timestamp& timestamp ) const;
			bool operator<=( const Timestamp& timestamp ) const;
			bool operator>=( const Timestamp& timestamp ) const;

			static Timestamp fromText( const std::string& strText );
			std::string asText() const;

			static const Timestamp now( Zone zone = local );
	};
};

#endif /* !__Base4c_Timestamp_h__ */
