/*
	$Id: Variant.h 1211 2018-11-28 20:09:51Z john $

	This file is part of the Base4c Foundation Classes.

	Base4c is Copyright (c) 2010 - 2018, The 4c Company.

	Base4c is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	Base4c is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __Base4c_Variant_h__
#define __Base4c_Variant_h__

#include <Base4c/Base4c.h>

#include <Base4c/Exception.h>

#include <Base4c/Timestamp.h>

#include <string>

namespace Base4c
{
	class Base4c_EXPORT Variant
	{
		public:
			enum VariantType{ varNone, varInt, varDouble, varString, varDate, varTime, varTimestamp, varBinary };

		private:
			VariantType _type;
			int _nValue;
			double _dValue;
			std::string _strValue;
			Base4c::Date _dtValue;
			Base4c::Time _tmValue;
			Base4c::Timestamp _tsValue;
			unsigned char* _pbyValue;
			unsigned int _uLen;
			bool _bNow;

		public:
			Variant();
			Variant( int nValue );
			Variant( double dValue );
			Variant( const std::string& strValue );
			Variant( const Base4c::Date& dtValue, bool bNow = false );
			Variant( const Base4c::Time& tmValue, bool bNow = false );
			Variant( const Base4c::Timestamp& tsValue, bool bNow = false );
			Variant( const unsigned char* pbyValue, unsigned int uLen );
			Variant( const Variant& var );
			virtual ~Variant();

			Variant& operator=( const Variant& var );

			VariantType type() const;
			int toInt() const;
			double toDouble() const;
			std::string toString() const;
			Base4c::Date toDate() const;
			Base4c::Time toTime() const;
			Base4c::Timestamp toTimestamp() const;
			const unsigned char* toBinary() const;
			unsigned int length() const;

			static Variant coerce( const Variant& value, VariantType type );

			static VariantType typeFromInt( int nType );
			static VariantType typeFromText( const std::string& strType );
	};
}

#endif /* !__Base4c_Variant_h__ */
