/*
	$Id: Version.h 1238 2018-12-03 14:46:43Z john $

	This file is part of the Base4c Foundation Classes.

	Base4c is Copyright (c) 2010 - 2018, The 4c Company.

	Base4c is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, version 2 of the License.

	Base4c is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with This code.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __Base4c_Version_h__
#define __Base4c_Version_h__

#include <Base4c/Base4c.h>

#include <Base4c/Exception.h>

#include <string>

namespace Base4c
{
	class Base4c_EXPORT Version
	{
		public:
#if defined (LINUX)
			typedef void* Handle;
#elif defined (_WINDOWS)
			typedef HMODULE Handle;
#endif /* LINUX */

		private:
			int _nVerMajor;
			int _nVerMinor;
			int _nRelease;
			int _nBuild;

		public:
			Version( int nVerMajor = 0, int nVerMinor = 0, int nRelease = 0, int nBuild = 0 );
			Version( const Version& version );
			virtual ~Version();

			const Version& operator=( const Version& version );

			bool operator==( const Version& version ) const;
			bool operator!=( const Version& version ) const;
			bool operator<=( const Version& version ) const;
			bool operator<( const Version& version ) const;
			bool operator>=( const Version& version ) const;
			bool operator>( const Version& version ) const;

			// For the range specified, compare returns 0 if the two versions are equal,
			// -1 if the passed version is less than *this and 1 if the passed version
			// is greater than *this.
			// The range specifies which members to check:
			// 0 = only the major version
			// 1 = the major and minor
			// 2 = major, minor and build
			// 3 = all values
			int compare( const Version& version, int nRange = 1 ) const;

			int verMajor() const;
			int verMinor() const;
			int release() const;
			int build() const;

			std::string text() const;
			std::string textLS() const;
			static Version fromText( const std::string& strText );

//			static const Version& version( const std::string& strSharedObject );
//			static const Version& version( Handle handle );
	};
};

extern "C" const Base4c::Version& getVersion();

#endif /* !__Base4c_Version_h__ */
